/*
 * ******************************************************************************************
 *
 * File		: apb_gpio.h
 * Author	: GOWIN
 * Function	: Definition of APB GPIO
 *
 * ******************************************************************************************
 */

#ifndef __APB_GPIO_H__
#define __APB_GPIO_H__


#ifdef __cplusplus
extern "C" {
#endif


// Includes ---------------------------------------------------------------------------------
#include <stddef.h>


/*****************************************************************************
 * Device Specific Peripheral Registers structures
 ****************************************************************************/
#define __RO          volatile const  /* 'read  only' permissions     */
#define __WO          volatile        /* 'write only' permissions     */
#define __RW          volatile        /* 'read/write' permissions     */


/*****************************************************************************
 * APB GPIO (General Purpose I/O)
 ****************************************************************************/
typedef struct
{
	__RO  unsigned int IDREV;                /* 0x00 ID and Revision Register        */
	      unsigned int RESERVED0[3];         /* 0x04~0x0C Reserved                   */
	__RO  unsigned int CFG;                  /* 0x10 Configuration Register          */
	      unsigned int RESERVED1[3];         /* 0x14~0x1C Reserved                   */
	__RO  unsigned int DATAIN;               /* 0x20 Channel Data-in Register        */
	__RW  unsigned int DATAOUT;              /* 0x24 Channel Data-out Register       */
	__RW  unsigned int CHANNELDIR;           /* 0x28 Channel Direction Register      */
	__WO  unsigned int DOUTCLEAR;            /* 0x2C Channel Data-out Clear Register */
	__WO  unsigned int DOUTSET;              /* 0x30 Channel Data-out Set Register   */
	      unsigned int RESERVED2[3];         /* 0x34~0x3C Reserved                   */
	__RW  unsigned int PULLEN;               /* 0x40 Pull Enable Register            */
	__RW  unsigned int PULLTYPE;             /* 0x44 Pull Type Register              */
	      unsigned int RESERVED3[2];         /* 0x48~0x4C Reserved                   */
	__RW  unsigned int INTREN;               /* 0x50 Interrupt Enable Register       */
	__RW  unsigned int INTRMODE0;            /* 0x54 Interrupt Mode Register (0~7)   */
	__RW  unsigned int INTRMODE1;            /* 0x58 Interrupt Mode Register (8~15)  */
	__RW  unsigned int INTRMODE2;            /* 0x5C Interrupt Mode Register (16~23) */
	__RW  unsigned int INTRMODE3;            /* 0x60 Interrupt Mode Register (24~31) */
	__RW  unsigned int INTRSTATUS;           /* 0x64 Interrupt Status Register       */
          unsigned int RESERVED4[2];         /* 0x68~0x6C Reserved                   */
	__RW  unsigned int DEBOUNCEEN;           /* 0x70 De-bounce Enable Register       */
	__RW  unsigned int DEBOUNCECTRL;         /* 0x74 De-bounce control register      */
	      unsigned int RESERVED5[2];         /* 0x78~0x7C Reserved                   */
} APB_GPIO_RegDef;


/****** APB GPIO Control Codes *****/

// APB GPIO direction
// Channel direction register
#define APB_GPIO_DIR_INPUT			  0x0			// Input direction
#define APB_GPIO_DIR_OUTPUT           0x1			// Output direction

// APB GPIO De-bounce clock source selection
#define APB_GPIO_DB_CLKSRC_EXT		  0x0			// External clock source
#define APB_GPIO_DB_CLKSRC_P		  0x1			// APB clock source

// Control command
// APB GPIO interrupt mode
#define APB_GPIO_INTR_ENABLE         	(1UL << 0)	// Enable interrupt
#define APB_GPIO_INTR_DISABLE           (1UL << 1)	// Disable interrupt
#define APB_GPIO_SET_INTR_LOW_LEVEL     (1UL << 2)	// Low level
#define	APB_GPIO_SET_INTR_HIGH_LEVEL    (1UL << 3)	// High level
#define	APB_GPIO_SET_INTR_NEGATIVE_EDGE (1UL << 4)	// Negative edge
#define	APB_GPIO_SET_INTR_POSITIVE_EDGE (1UL << 5)	// Positive edge
#define	APB_GPIO_SET_INTR_DUAL_EDGE     (1UL << 6)	// Dual edge
#define APB_GPIO_INTR_CLEAR             (1UL << 7)	// Clear interrupt status

//APB GPIO pull mode
#define APB_GPIO_PULL_ENABLE			(1UL << 8)	// Enable pull
#define APB_GPIO_PULL_DISABLE			(1UL << 9)	// Disable pull
#define APB_GPIO_SET_PULL_UP			(1UL << 10)	// Pull up
#define APB_GPIO_SET_PULL_DOWN		    (1UL << 11)	// Pull down

//APB GPIO de-bounce mode
#define APB_GPIO_DB_ENABLE              (1UL << 12)	// Enable de-bounce
#define APB_GPIO_DB_DISABLE             (1UL << 13)	// Disable de-bounce
#define APB_GPIO_SET_DB_CLKSRC		    (1UL << 14)	// De-bounce clock source
#define APB_GPIO_SET_DB_PRESCALE		(1UL << 15)	// De-bounce pre-scale

// APB GPIO interrupt signal event ID
#define	APB_GPIO_EVENT_PIN0             (1UL << 0)	// Pin 0 input interrupt event
#define	APB_GPIO_EVENT_PIN1             (1UL << 1)	// Pin 1 input interrupt event
#define	APB_GPIO_EVENT_PIN2             (1UL << 2)	// Pin 2 input interrupt event
#define	APB_GPIO_EVENT_PIN3             (1UL << 3)	// Pin 3 input interrupt event
#define	APB_GPIO_EVENT_PIN4             (1UL << 4)	// Pin 4 input interrupt event
#define APB_GPIO_EVENT_PIN5             (1UL << 5)	// Pin 5 input interrupt event
#define	APB_GPIO_EVENT_PIN6             (1UL << 6)	// Pin 6 input interrupt event
#define	APB_GPIO_EVENT_PIN7             (1UL << 7)	// Pin 7 input interrupt event
#define	APB_GPIO_EVENT_PIN8             (1UL << 8)	// Pin 8 input interrupt event
#define	APB_GPIO_EVENT_PIN9             (1UL << 9)	// Pin 9 input interrupt event
#define	APB_GPIO_EVENT_PIN10            (1UL << 10)	// Pin 10 input interrupt event
#define	APB_GPIO_EVENT_PIN11            (1UL << 11)	// Pin 11 input interrupt event
#define	APB_GPIO_EVENT_PIN12            (1UL << 12)	// Pin 12 input interrupt event
#define	APB_GPIO_EVENT_PIN13            (1UL << 13)	// Pin 13 input interrupt event
#define	APB_GPIO_EVENT_PIN14            (1UL << 14)	// Pin 14 input interrupt event
#define	APB_GPIO_EVENT_PIN15            (1UL << 15)	// Pin 15 input interrupt event
#define	APB_GPIO_EVENT_PIN16            (1UL << 16)	// Pin 16 input interrupt event
#define	APB_GPIO_EVENT_PIN17            (1UL << 17)	// Pin 17 input interrupt event
#define	APB_GPIO_EVENT_PIN18            (1UL << 18)	// Pin 18 input interrupt event
#define	APB_GPIO_EVENT_PIN19            (1UL << 19)	// Pin 19 input interrupt event
#define	APB_GPIO_EVENT_PIN20            (1UL << 20)	// Pin 20 input interrupt event
#define	APB_GPIO_EVENT_PIN21            (1UL << 21)	// Pin 21 input interrupt event
#define	APB_GPIO_EVENT_PIN22            (1UL << 22)	// Pin 22 input interrupt event
#define	APB_GPIO_EVENT_PIN23            (1UL << 23)	// Pin 23 input interrupt event
#define	APB_GPIO_EVENT_PIN24            (1UL << 24)	// Pin 24 input interrupt event
#define	APB_GPIO_EVENT_PIN25            (1UL << 25)	// Pin 25 input interrupt event
#define	APB_GPIO_EVENT_PIN26            (1UL << 26)	// Pin 26 input interrupt event
#define	APB_GPIO_EVENT_PIN27            (1UL << 27)	// Pin 27 input interrupt event
#define	APB_GPIO_EVENT_PIN28            (1UL << 28)	// Pin 28 input interrupt event
#define	APB_GPIO_EVENT_PIN29            (1UL << 29)	// Pin 29 input interrupt event
#define	APB_GPIO_EVENT_PIN30            (1UL << 30)	// Pin 30 input interrupt event
#define	APB_GPIO_EVENT_PIN31            (1UL << 31)	// Pin 31 input interrupt event


// Configuration register status
typedef struct
{
	unsigned int pull_opt      : 1;			// Pull option
	unsigned int intr_opt      : 1;			// Interrupt option
	unsigned int debounce_opt  : 1;			// De-bounce option
	unsigned int channel_num   : 6;			// Channel number
} APB_GPIO_CFG_STATUS;


// Callback function
typedef void (*APB_GPIO_SignalEvent_t) (unsigned int event);


/* General return codes */
#define APB_GPIO_DRIVER_OK                 0 // Operation succeeded
#define APB_GPIO_DRIVER_ERROR             -1 // Unspecified error
#define APB_GPIO_DRIVER_ERROR_BUSY        -2 // Driver is busy
#define APB_GPIO_DRIVER_ERROR_TIMEOUT     -3 // Timeout occurred
#define APB_GPIO_DRIVER_ERROR_UNSUPPORTED -4 // Operation not supported
#define APB_GPIO_DRIVER_ERROR_PARAMETER   -5 // Parameter error
#define APB_GPIO_DRIVER_ERROR_SPECIFIC    -6 // Start of driver specific errors

/****** APB GPIO specific error codes *****/
#define APB_GPIO_ERROR_MODE                (APB_GPIO_DRIVER_ERROR_SPECIFIC - 1)     // Specified mode not supported
#define APB_GPIO_ERROR_BAUDRATE            (APB_GPIO_DRIVER_ERROR_SPECIFIC - 2)     // Specified baud rate not supported
#define APB_GPIO_ERROR_DATA_BITS           (APB_GPIO_DRIVER_ERROR_SPECIFIC - 3)     // Specified number of data bits not supported
#define APB_GPIO_ERROR_PARITY              (APB_GPIO_DRIVER_ERROR_SPECIFIC - 4)     // Specified parity not supported
#define APB_GPIO_ERROR_STOP_BITS           (APB_GPIO_DRIVER_ERROR_SPECIFIC - 5)     // Specified number of stop bits not supported
#define APB_GPIO_ERROR_FLOW_CONTROL        (APB_GPIO_DRIVER_ERROR_SPECIFIC - 6)     // Specified flow control not supported
#define APB_GPIO_ERROR_CPOL                (APB_GPIO_DRIVER_ERROR_SPECIFIC - 7)     // Specified clock polarity not supported
#define APB_GPIO_ERROR_CPHA                (APB_GPIO_DRIVER_ERROR_SPECIFIC - 8)     // Specified clock phase not supported


// APB GPIO interrupt mode
#define APB_GPIO_INTER_NONE				0x0			// None
#define APB_GPIO_INTR_HIGH_LEVEL        0x2			// High level
#define APB_GPIO_INTR_LOW_LEVEL         0x3			// Low level
#define APB_GPIO_INTR_NEGATIVE_EDGE     0x5			// Negative edge
#define APB_GPIO_INTR_POSITIVE_EDGE     0x6			// Positive edge
#define APB_GPIO_INTR_DUAL_EDGE         0x7			// Dual edge

// APB GPIO pin number
#define APB_GPIO_MAX_PIN_NUM            32


// APB GPIO information
typedef struct _APB_GPIO_INFO
{
	APB_GPIO_SignalEvent_t cb_event;  		// Event callback
} APB_GPIO_INFO;

// APB GPIO resources
typedef struct
{
	APB_GPIO_RegDef						* reg;		// Pointer to APB GPIO register
	unsigned int						irq_num;	// APB GPIO IRQ number
	APB_GPIO_INFO						* info;		// Run-time information
} const APB_GPIO_RESOURCES;


#ifdef __cplusplus
}
#endif


#endif	/* __APB_GPIO_H__ */
