/*
 * ******************************************************************************************
 *
 * File		: apb_gpio_driver.c
 * Author	: GOWIN
 * Function	: Driver definition of APB GPIO
 *
 * ******************************************************************************************
 */


// Includes ---------------------------------------------------------------------------------
#include "apb_gpio_driver.h"
#include "apb_gpio_hal.h"


// Definitions ------------------------------------------------------------------------------

// ------------------------------------------------------------------------------------------
// Functions  : Set APB GPIO interrupt mode
// Parameters : intr_mode : APB GPIO interrupt mode
//              mask      : APB GPIO ports to be used
//				apb_gpiox : Pointer to APB GPIO resources
// ------------------------------------------------------------------------------------------
static void apb_gpio_set_intr_mode(unsigned int intr_mode, unsigned int mask, APB_GPIO_RESOURCES *apb_gpiox)
{
	unsigned int i;

	for (i = 0; i < APB_GPIO_MAX_PIN_NUM; i++)
	{
		if (mask & (0x1 << i))
		{
			if ((i >= 0) & (i < 8))
			{
				apb_gpiox->reg->INTRMODE0 &= ~(7 << (i * 4));					// Clear pin-0~7's interrupt mode
				apb_gpiox->reg->INTRMODE0 |= (intr_mode << (i * 4));			// Set pin-0~7's interrupt mode
			}
			else if ((i >= 8) & (i < 16))
			{
				apb_gpiox->reg->INTRMODE1 &= ~(7 << ((i-8) * 4));				// Clear pin-8~15's interrupt mode
				apb_gpiox->reg->INTRMODE1 |= (intr_mode << ((i-8) * 4));		// Set pin-8~15's interrupt mode
			}
			else if ((i >= 16) & (i < 24))
			{
				apb_gpiox->reg->INTRMODE2 &= ~(7 << ((i-16) * 4));				// Clear pin-16~23's interrupt mode
				apb_gpiox->reg->INTRMODE2 |= (intr_mode << ((i-16) * 4));		// Set pin-16~23's interrupt mode
			}
			else if ((i >= 24) & (i < 32))
			{
				apb_gpiox->reg->INTRMODE3 &= ~(7 << ((i-24) * 4));				// Clear pin-24~31's interrupt mode
				apb_gpiox->reg->INTRMODE3 |= (intr_mode << ((i-24) * 4));		// Set pin-24~31's interrupt mode
			}
		}
	}
}

// ------------------------------------------------------------------------------------------
// Functions  : Initializes APB GPIO
// Parameters : cb_event  : APB GPIO ports interrupt event
//				apb_gpiox : Pointer to APB GPIO resources
// Returns    : General return codes
// ------------------------------------------------------------------------------------------
int apb_gpio_initialize(APB_GPIO_SignalEvent_t cb_event, APB_GPIO_RESOURCES *apb_gpiox)
{
    // Disable all interrupts
	apb_gpiox->reg->INTREN = 0;

    // Write 1 to clear interrupt status
	apb_gpiox->reg->INTRSTATUS = 0xFFFFFFFF;

	// Enable APB GPIO interrupt
	hal_enable_interrupt(apb_gpiox->irq_num);

	// Interrupt event
	apb_gpiox->info->cb_event = cb_event;

	return APB_GPIO_DRIVER_OK;
}

// ------------------------------------------------------------------------------------------
// Functions  : Uninitialized APB GPIO
// Parameters : apb_gpiox : Pointer to APB GPIO resources
// Returns    : General return codes
// ------------------------------------------------------------------------------------------
int apb_gpio_uninitialize(APB_GPIO_RESOURCES *apb_gpiox)
{
	// Disable all interrupts
	apb_gpiox->reg->INTREN = 0;

    // Write 1 to clear interrupt status
	apb_gpiox->reg->INTRSTATUS = 0xFFFFFFFF;

	// Disable APB GPIO interrupt
	hal_disable_interrupt(apb_gpiox->irq_num);

	apb_gpiox->info->cb_event = NULL;

	return APB_GPIO_DRIVER_OK;
}

// ------------------------------------------------------------------------------------------
// Functions  : Write a value into a pin
// Parameters : pin_num   : A APB GPIO pin
//              val       : The value written into the pin
//				apb_gpiox : Pointer to APB GPIO resources
// ------------------------------------------------------------------------------------------
void apb_gpio_pinwrite(unsigned int pin_num, int val, APB_GPIO_RESOURCES *apb_gpiox)
{
	val ? (apb_gpiox->reg->DOUTSET = 1UL << pin_num) : (apb_gpiox->reg->DOUTCLEAR = (1UL << pin_num));
}

// ------------------------------------------------------------------------------------------
// Functions  : Read a value from a pin
// Parameters : pin_num   : A APB GPIO pin
//				apb_gpiox : Pointer to APB GPIO resources
// Returns    : The value read from the pin
// ------------------------------------------------------------------------------------------
unsigned char apb_gpio_pinread(unsigned int pin_num, APB_GPIO_RESOURCES *apb_gpiox)
{
	return ((apb_gpiox->reg->DATAIN & (1UL << pin_num)) ? (1) : (0));
}

// ------------------------------------------------------------------------------------------
// Functions  : Write a value into APB GPIO ports
// Parameters : mask      : APB GPIO ports to be used
//              val       : The value written into APB GPIO ports
//				apb_gpiox : Pointer to APB GPIO resources
// ------------------------------------------------------------------------------------------
void apb_gpio_write(unsigned int mask, unsigned int val, APB_GPIO_RESOURCES *apb_gpiox)
{
	val ? (apb_gpiox->reg->DOUTSET = mask) : (apb_gpiox->reg->DOUTCLEAR = mask);
}

// ------------------------------------------------------------------------------------------
// Functions  : Read a value from APB GPIO ports
// Parameters : apb_gpiox : Pointer to APB GPIO resources
// Returns    : The value read from APB GPIO ports
// ------------------------------------------------------------------------------------------
unsigned int apb_gpio_read(APB_GPIO_RESOURCES *apb_gpiox)
{
	return (apb_gpiox->reg->DATAIN);
}

// ------------------------------------------------------------------------------------------
// Functions  : Set APB GPIO direction
// Parameters : mask      : APB GPIO ports to be used
//              dir       : Direction input/output
//				apb_gpiox : Pointer to APB GPIO resources
// ------------------------------------------------------------------------------------------
void apb_gpio_setdir(unsigned int mask, int dir, APB_GPIO_RESOURCES *apb_gpiox)
{
	dir ? (apb_gpiox->reg->CHANNELDIR |= mask) : (apb_gpiox->reg->CHANNELDIR &= ~mask);
}

// ------------------------------------------------------------------------------------------
// Functions  : Control command for APB GPIO interrupt, pull, de-bounce
// Parameters : mode      : Interrupt, pull, de-bounce mode or enable/disable
//              mask      : APB GPIO ports to be used
//				clksel    : De-bounce clock source selection (extclk/pclk)
//				scale     : APB GPIO pre-scale base
//				apb_gpiox : Pointer to APB GPIO resources
// Returns    : General return code
// ------------------------------------------------------------------------------------------
int apb_gpio_control(unsigned int mode, unsigned int mask, unsigned int clksel, unsigned int scale, APB_GPIO_RESOURCES *apb_gpiox)
{
	/* Interrupt --------------------------------------------------------------- */
	// Set interrupt mode
	if (mode & APB_GPIO_SET_INTR_HIGH_LEVEL)
	{
		apb_gpio_set_intr_mode(APB_GPIO_INTR_HIGH_LEVEL, mask, apb_gpiox);
	}
	else if (mode & APB_GPIO_SET_INTR_LOW_LEVEL)
	{
		apb_gpio_set_intr_mode(APB_GPIO_INTR_LOW_LEVEL, mask, apb_gpiox);
	}
	else if (mode & APB_GPIO_SET_INTR_NEGATIVE_EDGE)
	{
		apb_gpio_set_intr_mode(APB_GPIO_INTR_NEGATIVE_EDGE, mask, apb_gpiox);
	}
	else if (mode & APB_GPIO_SET_INTR_POSITIVE_EDGE)
	{
		apb_gpio_set_intr_mode(APB_GPIO_INTR_POSITIVE_EDGE, mask, apb_gpiox);
	}
	else if (mode & APB_GPIO_SET_INTR_DUAL_EDGE)
	{
		apb_gpio_set_intr_mode(APB_GPIO_INTR_DUAL_EDGE, mask, apb_gpiox);
	}

	// Interrupt disable/enable
	if (mode & APB_GPIO_INTR_DISABLE)
	{
		apb_gpiox->reg->INTREN &= ~mask;
	}
	else if (mode & APB_GPIO_INTR_ENABLE)
	{
		apb_gpiox->reg->INTREN |= mask;
	}

	// Interrupt clear status
	if (mode & APB_GPIO_INTR_CLEAR)
	{
		apb_gpiox->reg->INTRSTATUS |= mask;
	}

	/* Pull ------------------------------------------------------------------- */
	// Pull disable/enable
	if (mode & APB_GPIO_PULL_DISABLE)
	{
		apb_gpiox->reg->PULLEN &= ~mask;			// 0x0 : disable pull
	}
	else if (mode & APB_GPIO_PULL_ENABLE)
	{
		apb_gpiox->reg->PULLEN |= mask;				// 0x1 : enable pull
	}

	// Pull type
	if (mode & APB_GPIO_SET_PULL_UP)
	{
		apb_gpiox->reg->PULLTYPE &= ~mask;			// 0x0 : pull up
	}
	else if (mode & APB_GPIO_SET_PULL_DOWN)
	{
		apb_gpiox->reg->PULLTYPE |= mask;			// 0x1 : pull down
	}

	/* De-bounce --------------------------------------------------------------- */
	// De-bounce disable/enable
	if (mode & APB_GPIO_DB_DISABLE)
	{
		apb_gpiox->reg->DEBOUNCEEN &= ~mask;		// 0x0 : disable de-bounce
	}
	else if (mode & APB_GPIO_DB_ENABLE)
	{
		apb_gpiox->reg->DEBOUNCEEN |= mask;			// 0x1 : enable de-bounce
	}

	// De-bounce clock source
	if (mode & APB_GPIO_SET_DB_CLKSRC)
	{
		apb_gpiox->reg->DEBOUNCECTRL &= 0x7FFFFFFF;						// Clear [31] to 0
		//clksel : APB_GPIO_DB_CLKSRC_EXT or APB_GPIO_DB_CLKSRC_P
		apb_gpiox->reg->DEBOUNCECTRL |= ((clksel << 31) & 0x80000000);	// Set [31] to clksel (extclk or pclk)
	}

	// De-bounce pre-scale
	if (mode & APB_GPIO_SET_DB_PRESCALE)
	{
		apb_gpiox->reg->DEBOUNCECTRL &= 0xFFFFFF00;						// Clear [7:0] to 0
		apb_gpiox->reg->DEBOUNCECTRL |= (scale & 0x000000FF);			// Set [7:0] to scale
	}

	return APB_GPIO_DRIVER_OK;
}

// ------------------------------------------------------------------------------------------
// Functions  : Get APB GPIO configuration status
// Parameters : cfg_info  : Configuration status
//				apb_gpiox : Pointer to APB GPIO resources
// Returns    : General return codes
// ------------------------------------------------------------------------------------------
int apb_gpio_get_status(APB_GPIO_CFG_STATUS* cfg_info, APB_GPIO_RESOURCES *apb_gpiox)
{
	unsigned int cfg_reg = apb_gpiox->reg->CFG;

	// Get pull option
	cfg_info->pull_opt = (cfg_reg & 0x80000000) >> 31;			// [31] pull option

	// Get interrupt option
	cfg_info->intr_opt = (cfg_reg & 0x40000000) >> 30;			// [30] interrupt option

	// Get de-bounce option
	cfg_info->debounce_opt = (cfg_reg & 0x20000000) >> 29;		// [29] de-bounce option

	// Get channel number
	cfg_info->channel_num = cfg_reg & 0x0000003F;				// [5:0] channel number

	return APB_GPIO_DRIVER_OK;
}

// ------------------------------------------------------------------------------------------
// Functions  : APB GPIO interrupt handler
// Parameters : apb_gpiox : Pointer to APB GPIO resources
// ------------------------------------------------------------------------------------------
void apb_gpio_irq_handler(APB_GPIO_RESOURCES *apb_gpiox)
{
	// Trigger interrupt by event
	unsigned int status;
	unsigned int i;

	status = apb_gpiox->reg->INTRSTATUS;

	// Write 1 to clear interrupt status
	apb_gpiox->reg->INTRSTATUS = status;

	for (i = 0; i < APB_GPIO_MAX_PIN_NUM; i++)
	{
		// If a pin has an interrupt, trigger interrupt event
		if (status & (0x1 << i))
		{
			apb_gpiox->info->cb_event((1UL << i));
		}
	}
}
