
// APB GPIO demo
module apb_gpio_demo
(
    input CLK,
    input RSTN,
    // LED
    inout [2:0] LED,
    // SPI Flash Memory
    inout FLASH_SPI_CSN,
    inout FLASH_SPI_MISO,
    inout FLASH_SPI_MOSI,
    inout FLASH_SPI_CLK,
    inout FLASH_SPI_HOLDN,
    inout FLASH_SPI_WPN,
    // DDR3 Memory
    output DDR3_INIT,
    output [2:0] DDR3_BANK,
    output DDR3_CS_N,
    output DDR3_RAS_N,
    output DDR3_CAS_N,
    output DDR3_WE_N,
    output DDR3_CK,
    output DDR3_CK_N,
    output DDR3_CKE,
    output DDR3_RESET_N,
    output DDR3_ODT,
    output [13:0] DDR3_ADDR,
    output [1:0] DDR3_DM,
    inout [15:0] DDR3_DQ,
    inout [1:0] DDR3_DQS,
    inout [1:0] DDR3_DQS_N,
    // JTAG
    input TCK_IN,
    input TMS_IN,
    input TRST_IN,
    input TDI_IN,
    output TDO_OUT
);


wire [2:0] gpio_in;
wire [2:0] gpio_oe;
wire [2:0] gpio_out;

// GPIO inout
gw_io gpio_0_io (.O(gpio_in[0]), .I(gpio_out[0]), .IO(LED[0]), .E(gpio_oe[0]), .PU(1'b0), .PD(1'b0));
gw_io gpio_1_io	(.O(gpio_in[1]), .I(gpio_out[1]), .IO(LED[1]), .E(gpio_oe[1]), .PU(1'b0), .PD(1'b0));
gw_io gpio_2_io	(.O(gpio_in[2]), .I(gpio_out[2]), .IO(LED[2]), .E(gpio_oe[2]), .PU(1'b0), .PD(1'b0));


wire CORE_CLK;
wire DDR_CLK;
wire AHB_CLK;
wire APB_CLK;
wire RTC_CLK;

wire GPIO_CLK;

wire DDR3_MEMORY_CLK;
wire DDR3_CLK_IN;
wire DDR3_LOCK;
wire DDR3_STOP;


// Gowin_PLL_AE350 instantiation
Gowin_PLL_AE350 u_Gowin_PLL_AE350
(
    .clkout0(DDR_CLK),          // 50MHz  - for test
    .clkout1(CORE_CLK),         // 800MHz
    .clkout2(AHB_CLK),          // 50MHz  - for test
    .clkout3(APB_CLK),          // 50MHz  - for test
    .clkout4(RTC_CLK),          // 10MHz
    .clkout5(GPIO_CLK),         // 10MHz
                                // This clock source must be at least 4x slower than APB_CLK.
    .clkin(CLK),
    .init_clk(CLK),
    .reset(1'b0),
    .enclk0(1'b1),
    .enclk1(1'b1),
    .enclk2(1'b1),
    .enclk3(1'b1),
    .enclk4(1'b1),
    .enclk5(1'b1)
);


// Gowin_PLL_DDR3 instantiation
Gowin_PLL_DDR3 u_Gowin_PLL_DDR3
(
    .lock(DDR3_LOCK),
    .clkout0(DDR3_CLK_IN),          // 50MHz
    .clkout2(DDR3_MEMORY_CLK),      // 200MHz - for test
    .clkin(CLK),
    .init_clk(CLK),
    .reset(1'b0),
    .enclk0(1'b1),
    .enclk2(DDR3_STOP)
);


wire ae350_rstn;                    // AE350 power on and hardware reset in
wire ddr3_rstn;                     // DDR3 memory reset in
wire ddr3_init_completed;           // DDR3 memory initialized completed

assign DDR3_INIT = ddr3_init_completed;


// key_debounce instantiation
// DDR3 memory reset in key debounce
key_debounce u_key_debounce_ddr3
(
    .out(ddr3_rstn),
    .in(RSTN),
    .clk(CLK),      // 50MHz
    .rstn(1'b1)
);


// key_debounce instantiation
// AE350 power on and hardware reset in key debounce
key_debounce u_key_debounce_ae350
(
    .out(ae350_rstn),
    .in(ddr3_init_completed),
    .clk(CLK),      // 50MHz
    .rstn(1'b1)
);


// APB bus
wire [31:0]         APB_PRDATA;
wire                APB_PREADY;
wire                APB_PSLVERR;
wire [31:0]         APB_PADDR;
wire                APB_PENABLE;
wire                APB_PSEL;
wire [31:0]         APB_PWDATA;
wire                APB_PWRITE;
wire [2:0]          APB_PPROT;
wire [3:0]          APB_PSTRB;
wire                APB_PCLK;
wire                APB_PRSTN;

wire                gpio_intr;


// APB_GPIO_Top instantiation
APB_GPIO_Top u_APB_GPIO_Top
(
	.gpio_intr(gpio_intr),
	.gpio_oe(gpio_oe),
	.gpio_out(gpio_out),
    .gpio_in(gpio_in),
    .extclk(GPIO_CLK),      // This clock source must be at least 4x slower than pclk.
	.paddr(APB_PADDR[7:0]),
	.penable(APB_PENABLE),
	.prdata(APB_PRDATA),
	.psel(APB_PSEL),
	.pwdata(APB_PWDATA),
	.pwrite(APB_PWRITE),
	.pclk(APB_PCLK),
	.presetn(APB_PRSTN),
    .pready(APB_PREADY),
    .pslverr(APB_PSLVERR)
);


// RiscV_AE350_SOC_Top instantiation
RiscV_AE350_SOC_Top u_RiscV_AE350_SOC_Top
(
    .FLASH_SPI_CSN(FLASH_SPI_CSN),
    .FLASH_SPI_MISO(FLASH_SPI_MISO),
    .FLASH_SPI_MOSI(FLASH_SPI_MOSI),
    .FLASH_SPI_CLK(FLASH_SPI_CLK),
    .FLASH_SPI_HOLDN(FLASH_SPI_HOLDN),
    .FLASH_SPI_WPN(FLASH_SPI_WPN),
    .DDR3_MEMORY_CLK(DDR3_MEMORY_CLK),
    .DDR3_CLK_IN(DDR3_CLK_IN),
    .DDR3_RSTN(ddr3_rstn),              // DDR3 memory reset in, 0 is reset state
    .DDR3_LOCK(DDR3_LOCK),
    .DDR3_STOP(DDR3_STOP),
    .DDR3_INIT(ddr3_init_completed),    // DDR3 memory initialized completed
    .DDR3_BANK(DDR3_BANK),
    .DDR3_CS_N(DDR3_CS_N),
    .DDR3_RAS_N(DDR3_RAS_N),
    .DDR3_CAS_N(DDR3_CAS_N),
    .DDR3_WE_N(DDR3_WE_N),
    .DDR3_CK(DDR3_CK),
    .DDR3_CK_N(DDR3_CK_N),
    .DDR3_CKE(DDR3_CKE),
    .DDR3_RESET_N(DDR3_RESET_N),
    .DDR3_ODT(DDR3_ODT),
    .DDR3_ADDR(DDR3_ADDR),
    .DDR3_DM(DDR3_DM),
    .DDR3_DQ(DDR3_DQ),
    .DDR3_DQS(DDR3_DQS),
    .DDR3_DQS_N(DDR3_DQS_N),
    .TCK_IN(TCK_IN),
    .TMS_IN(TMS_IN),
    .TRST_IN(TRST_IN),
    .TDI_IN(TDI_IN),
    .TDO_OUT(TDO_OUT),
    .TDO_OE(),
    .EXT_INT({15'b0, gpio_intr}),
    .APB_PRDATA(APB_PRDATA),
    .APB_PREADY(APB_PREADY),
    .APB_PSLVERR(APB_PSLVERR),
    .APB_PADDR(APB_PADDR),
    .APB_PENABLE(APB_PENABLE),
    .APB_PSEL(APB_PSEL),
    .APB_PWDATA(APB_PWDATA),
    .APB_PWRITE(APB_PWRITE),
    .APB_PPROT(APB_PPROT),
    .APB_PSTRB(APB_PSTRB),
    .APB_PCLK(APB_PCLK),
    .APB_PRSTN(APB_PRSTN),
    .CORE_CLK(CORE_CLK),
    .DDR_CLK(DDR_CLK),
    .AHB_CLK(AHB_CLK),
    .APB_CLK(APB_CLK),
    .RTC_CLK(RTC_CLK),
    .POR_RSTN(ae350_rstn),              // AE350 CPU core power on reset, 0 is reset state
    .HW_RSTN(ae350_rstn)                // AE350 hardware reset, 0 is reset state
);

endmodule


// gw_io
module gw_io (O, I, IO, E, PU, PD);
output  O;
input   I;
inout   IO;
input   E;
input   PU;
input   PD;

assign  IO	= E ? I : 1'bz;
assign  O	= IO;
assign  (pull0, pull1) IO = PU ? 1'b1 :
                            PD ? 1'b0 : 1'bz;

endmodule